@@ -27,6 +27,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -77,7 +80,6 @@
 #include <libxml/nanohttp.h>
 #include <libxml/globals.h>
 #include <libxml/uri.h>
-#include <config.h>
 
 /**
  * A couple portability macros
@@ -151,7 +153,8 @@ static int socket_errno(void) {
 }
 
 #ifdef SUPPORT_IP6
-static int have_ipv6() {
+static
+int have_ipv6(void) {
     int s;
 
     s = socket (AF_INET6, SOCK_STREAM, 0);
@@ -953,7 +956,7 @@ static int
 xmlNanoHTTPConnectHost(const char *host, int port)
 {
     struct hostent *h;
-    struct sockaddr *addr;
+    struct sockaddr *addr = NULL;
     struct in_addr ia;
     struct sockaddr_in sockin;
 
@@ -993,14 +996,13 @@ xmlNanoHTTPConnectHost(const char *host, int port)
 	}
 
 	for (res = result; res; res = res->ai_next) {
-	    if (res) {
+	    if (res->ai_family == AF_INET || res->ai_family == AF_INET6) {
 		if (res->ai_family == AF_INET6) {
 		    memcpy (&sockin6, res->ai_addr, res->ai_addrlen);
 		    sockin6.sin6_port = htons (port);
 		    addr = (struct sockaddr *)&sockin6;
 		}
-
-		if (res->ai_family == AF_INET) {
+		else {
 		    memcpy (&sockin, res->ai_addr, res->ai_addrlen);
 		    sockin.sin_port = htons (port);
 		    addr = (struct sockaddr *)&sockin;
@@ -1012,11 +1014,10 @@ xmlNanoHTTPConnectHost(const char *host, int port)
 		    return (s);
 		}
 	    }
-	    else {
-		freeaddrinfo (result);
-		return (-1);
-	    }
 	}
+	if (result)
+	    freeaddrinfo (result);
+	return (-1);
     } else
 #endif
 #endif
@@ -1213,9 +1214,11 @@ xmlNanoHTTPMethodRedir(const char *URL, const char *method, const char *input,
     char *bp, *p;
     int blen, ret;
     int head;
-    int xmt_bytes;
     int nbRedirects = 0;
     char *redirURL = NULL;
+#ifdef DEBUG_HTTP
+    int xmt_bytes;
+#endif
     
     if (URL == NULL) return(NULL);
     if (method == NULL) method = "GET";
@@ -1327,26 +1330,30 @@ retry:
     ctxt->outptr = ctxt->out = bp;
     ctxt->state = XML_NANO_HTTP_WRITE;
     blen = strlen( ctxt->out );
-    xmt_bytes = xmlNanoHTTPSend(ctxt, ctxt->out, blen );
 #ifdef DEBUG_HTTP
+    xmt_bytes = xmlNanoHTTPSend(ctxt, ctxt->out, blen );
     if ( xmt_bytes != blen )
         xmlGenericError( xmlGenericErrorContext,
 			"xmlNanoHTTPMethodRedir:  Only %d of %d %s %s\n",
 			xmt_bytes, blen,
 			"bytes of HTTP headers sent to host",
 			ctxt->hostname );
+#else
+    xmlNanoHTTPSend(ctxt, ctxt->out, blen );
 #endif
 
     if ( input != NULL ) {
+#ifdef DEBUG_HTTP
         xmt_bytes = xmlNanoHTTPSend( ctxt, input, ilen );
 
-#ifdef DEBUG_HTTP
 	if ( xmt_bytes != ilen )
 	    xmlGenericError( xmlGenericErrorContext,
 	    		"xmlNanoHTTPMethodRedir:  Only %d of %d %s %s\n",
 			xmt_bytes, ilen,
 			"bytes of HTTP content sent to host",
 			ctxt->hostname );
+#else
+	xmlNanoHTTPSend( ctxt, input, ilen );
 #endif
     }
 
