@@ -41,7 +41,9 @@ static char *filename;
 
 #ifdef LIBXML_CATALOG_ENABLED
 
+#ifndef XML_SGML_DEFAULT_CATALOG
 #define XML_SGML_DEFAULT_CATALOG "/etc/sgml/catalog"
+#endif
 
 /************************************************************************
  * 									*
@@ -192,7 +194,7 @@ static void usershell(void) {
 		if (ans == NULL) {
 		    printf("No entry for PUBLIC %s\n", argv[0]);
 		} else {
-		    printf("%s\n", ans);
+		    printf("%s\n", (char *) ans);
 		    xmlFree(ans);
 		}
 	    }
@@ -204,7 +206,7 @@ static void usershell(void) {
 		if (ans == NULL) {
 		    printf("No entry for SYSTEM %s\n", argv[0]);
 		} else {
-		    printf("%s\n", ans);
+		    printf("%s\n", (char *) ans);
 		    xmlFree(ans);
 		}
 	    }
@@ -254,7 +256,7 @@ static void usershell(void) {
 		if (ans == NULL) {
 		    printf("Resolver failed to find an answer\n");
 		} else {
-		    printf("%s\n", ans);
+		    printf("%s\n", (char *) ans);
 		    xmlFree(ans);
 		}
 	    }
@@ -537,7 +539,7 @@ int main(int argc, char **argv) {
 		    printf("No entry for PUBLIC %s\n", argv[i]);
 		    exit_value = 4;
 		} else {
-		    printf("%s\n", ans);
+		    printf("%s\n", (char *) ans);
 		    xmlFree(ans);
 		}
 	    } else {
@@ -547,7 +549,7 @@ int main(int argc, char **argv) {
 		    printf("No entry for SYSTEM %s\n", argv[i]);
 		    exit_value = 4;
 		} else {
-		    printf("%s\n", ans);
+		    printf("%s\n", (char *) ans);
 		    xmlFree(ans);
 		}
 	    }
